/* Copyright 2013 Andrey Dubovik <andrei.dubovik@gmail.com>
 *
 * This file is part of Minido.
 *
 * Minido is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Minido is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Minido. If not, see <http://www.gnu.org/licenses/>.
 */

#include "utilities.hpp"
#include "unicode.hpp"

using namespace std;

void sanitize(string& str) {
  replace<string>(str, "&", "&amp;");
  replace<string>(str, "<", "&lt;");
  replace<string>(str, ">", "&gt;");
}

void sanitize(u32string& str) {
  replace<u32string>(str, U"&", U"&amp;");
  replace<u32string>(str, U"<", U"&lt;");
  replace<u32string>(str, U">", U"&gt;");
}

string decorate(const string& str, const string& style) {
  auto newstr = str;
  sanitize(newstr);
  newstr = "<span " + style + ">" + newstr + "</span>";
  return newstr;
}

string decorate(const u32string& str, const string& style) {
  return decorate(utf32to8(str), style);
}

string decorate(const u32string& str, const string& style, const MatchMask& mask) {
  string newstr;
  int state = 0;
  int pos = 0;
  for(int i = 0; i < str.size(); ++i) {
    if(mask[i] != state) {
      u32string chunk(str.begin()+pos, str.begin()+i);
      sanitize(chunk);
      newstr += utf32to8(chunk);
      pos = i;
    }
    if(mask[i] > state) {
      newstr += "<span " + style + ">";
      state = 1;
    }
    if(mask[i] < state) {
      newstr += "</span>";
      state = 0;
    }
  }
  u32string chunk(str.begin()+pos, str.end());
  sanitize(chunk);
  newstr += utf32to8(chunk);
  if(state) newstr += "</span>";
  return newstr;
}
